@@ -248,7 +248,7 @@ read_name (void *arg, int argc, char **argv, char **names)
 {
   struct person *p = (struct person *)arg;
 
-  if (strcasecmp (argv[0], "NAME") == 0)
+  //if (strcasecmp (argv[0], "NAME") == 0)
     p->name = g_strdup (argv[1]);
 
   return 0;
@@ -261,17 +261,18 @@ read_one_entry (void *arg, int argc, char **argv, char **names)
   GSList **list = (GSList **) arg;
 
   p->id = atoi (argv[0]);
-
+/*
 #ifdef USE_USQLD	
-  usqld_exec_printf (db, "select tag,value from contacts where urn=%d",
+  usqld_exec_printf (db, "select tag,value from contacts where (urn=%d) and ((tag='NAME') or (tag='name'))",
 		     read_name, p, NULL, p->id);
 #else
-  sqlite_exec_printf (db, "select tag,value from contacts where urn=%d",
+  sqlite_exec_printf (db, "select tag,value from contacts where (urn=%d) and ((tag='NAME') or (tag='name'))",
 		      read_name, p, NULL, p->id);
 #endif  
-
+*/
+  p->name = g_strdup (argv[1]);
   if (p->name)
-    *list = g_slist_insert_sorted (*list, p, (GCompareFunc) sort_entries);
+    *list = g_slist_prepend (*list, p);
   else
     discard_person (p);
 
@@ -416,7 +417,7 @@ void
 db_set_multi_data (struct person *p, gchar * tag, gchar * value)
 {
   struct tag_value *t = new_tag_value (g_strdup (tag), value);
-  p->data = g_slist_append (p->data, t);
+  p->data = g_slist_prepend (p->data, t);
 }
 
 void
@@ -431,7 +432,7 @@ db_set_data (struct person *p, gchar * tag, gchar * value)
   else
     {
       t = new_tag_value (g_strdup (tag), value);
-      p->data = g_slist_append (p->data, t);
+      p->data = g_slist_prepend (p->data, t);
     }
 }
 
@@ -601,7 +602,7 @@ load_one_attribute (void *arg, int argc, char **argv, char **names)
       c->id = atoi (argv[0]);
       c->name = g_strdup (argv[1]);
 
-      *list = g_slist_append (*list, c);
+      *list = g_slist_prepend (*list, c);
     }
 
   return 0;
@@ -633,25 +634,54 @@ GSList *
 db_get_entries_alpha (const gchar * alphalist)
 {
   GSList *list = NULL;
-  GSList *all_entries = db_get_entries ();
-  GSList *i;
 
-  for (i = all_entries; i; i = i->next)
+  char *command, *tmp, *modifier;
+  char *err;
+  int r, s, t;
+
+  s = strlen(alphalist);
+  command = g_strdup("");
+	
+  if (alphalist[0] == '!')
+  {
+    t = 1;
+	modifier = g_strdup(" NOT ");
+  }	  
+  else
+  {
+	t = 0;  
+	modifier = g_strdup(" ");
+  }
+  for (r=t;r<s;r++)
+  {
+     tmp = g_strdup_printf("%s(value like '%c%%') or (value like '%c%%')",
+	   command,g_ascii_toupper(alphalist[r]),g_ascii_tolower(alphalist[r]));
+	 free(command); 
+	 if (r != s-1)
+	   command = g_strdup_printf("%s or",tmp);					 
+  }
+
+  command = g_strdup_printf("select urn, value from  \
+  		contacts where ((( tag = 'NAME') or ( tag = 'name')) and%s(%s))",modifier,tmp);
+  free(tmp);
+#ifdef USE_USQLD	
+  r = usqld_exec (db, command,
+		   read_one_entry, &list, &err);
+#else
+  r = sqlite_exec (db, command,
+		   read_one_entry, &list, &err);
+#endif
+
+  if (r)
     {
-      struct person *p = i->data;
-      
-      if (strchr (alphalist, p->name[0]))
-	list = g_slist_append (list, p);
-      else
-	discard_person (p);
+      gpe_error_box (err);
+      free (err);
+      return NULL;
     }
 
-  g_slist_free (all_entries);
-
   return list;
 }
 
-
 gint
 db_get_tag_list (gchar *** list)
 {
